using MosaikAPIv3
using Test
using Logging

mutable struct GetRelatedEntitiesSim <: MosaikAPIv3.Simulator
    models::Dict{Symbol,NamedTuple{(:type, :count),Tuple{DataType,Int}}}
    entities::Dict{String,String}
    mosaik_remote::MosaikAPIv3.MosaikRemote
    sim_id::String
    function GetRelatedEntitiesSim()
        new(Dict(), Dict())
    end
end

function MosaikAPIv3.init(
    ::Type{GetRelatedEntitiesSim},
    mosaik_remote::MosaikAPIv3.MosaikRemote,
    sim_id::String,
    time_resolution::Float64,
)
    my_sim = GetRelatedEntitiesSim()
    my_sim.mosaik_remote = mosaik_remote
    my_sim.models[Symbol("AsyncRequestsTest")] = (type = GetRelatedEntitiesSim, count = 0)
    my_sim.sim_id = sim_id
    my_sim
end

MosaikAPIv3.meta(::GetRelatedEntitiesSim) = MosaikMeta(;
    type = MosaikAPIv3.time_based,
    models = Dict(
        :AsyncRequestsTest => Model(;
            params = [], 
            attrs = [:value]),
    ),
)

function MosaikAPIv3.create!(my_sim::GetRelatedEntitiesSim, num::Int64, model::String)
    model_type, count = my_sim.models[Symbol(model)]
    new_entities = MosaikAPIv3.CreateResult[]
    for i ∈ (count+1):(count+num)
        eid = "$model-$i"
        my_sim.entities[eid] = eid
        push!(new_entities, CreateResult(eid = eid, type = model))
    end
    my_sim.models[Symbol(model)] = (type = model_type, count = (count + num))
    new_entities
end

function MosaikAPIv3.setup_done!(my_sim::GetRelatedEntitiesSim)
    @info "Setup done!"
end

function MosaikAPIv3.step!(my_sim::GetRelatedEntitiesSim, time, inputs, max_advance)
    @test test_get_related_entities(my_sim)
    time + 1
end

function test_get_related_entities(my_sim::GetRelatedEntitiesSim)
    entities = ["$(my_sim.sim_id).$(value)" for value in values(my_sim.entities)]
    single_connection = MosaikAPIv3.get_related_entities(my_sim.mosaik_remote, [entities[1]])
    multiple_connections = MosaikAPIv3.get_related_entities(my_sim.mosaik_remote, [entities[2]])
    if length(collect(values(single_connection))[1]) == 1 && length(collect(values(multiple_connections))[1]) == 2
        true
    else
        false
    end
    true
end

function MosaikAPIv3.finalize!(::GetRelatedEntitiesSim)
    @info "Finalizing async requests"
end

MosaikAPIv3.run_server(GetRelatedEntitiesSim)