using MosaikAPIv3
using Test
using Logging

global CONSTANT_KEY = :Constant

@kwdef mutable struct InputSim <: MosaikAPIv3.Simulator
    functions::Dict{String,Function} = Dict()
    constants::Dict{String,Any} = Dict()
    mosaik_remote::MosaikAPIv3.MosaikRemote
end

"""
init!(::Type{<:Simulator}, mosaik_remote::MosaikRemote, sim_id::String, time_resolution::Float64)

Initialize an input simulator instance.

# Arguments
- `mosaik_remote::MosaikRemote`: MosaikRemote object for communication with the Mosaik coordinator.
- `sim_id::String`: Unique identifier for the simulator instance.
- `time_resolution::Float64`: Time resolution of the simulator.
"""
function MosaikAPIv3.init(
    ::Type{InputSim},
    mosaik_remote::MosaikAPIv3.MosaikRemote,
    sim_id::String,
    time_resolution::Float64,
)
    @info "I'm simulator $sim_id, running with time resolution $time_resolution."
    InputSim(;
        mosaik_remote = mosaik_remote,
    )
end

MosaikAPIv3.meta(::InputSim) = MosaikMeta(;
    type = MosaikAPIv3.time_based,
    models = Dict(
        CONSTANT_KEY => Model(;
            params = [:constant],
            attrs = [:value],
        ),
    ),
)

function MosaikAPIv3.create!(sim::InputSim, num::Int64, model::String; model_params...)
    new_entities = MosaikAPIv3.CreateResult[]
    for i = length(sim.constants)+1:length(sim.constants)+num
        push!(new_entities, create_constant_entity(sim, i, model, model_params...))
    end
    return new_entities
end

function create_constant_entity(sim::InputSim, id::Int, model::String, constant::Any)
    eid = "$CONSTANT_KEY-$id"
    sim.constants[eid] = constant
    return MosaikAPIv3.CreateResult(eid = eid, type = model)
end

function MosaikAPIv3.setup_done!(::InputSim)
    @info "Setup done!"
end

function MosaikAPIv3.step!(::InputSim, time::Int, inputs, max_advance::Int)
    time + 1
end

function MosaikAPIv3.get_data(
    sim::InputSim,
    outputs::MosaikAPIv3.OutputRequest,
)::MosaikAPIv3.OutputData
    data = MosaikAPIv3.OutputData()
    for eid in keys(outputs)
        data[eid] = Dict("value" => sim.constants[eid])
    end
    return data
end
MosaikAPIv3.run_server(InputSim)