using MosaikAPIv3
using Test
using Logging

@kwdef mutable struct OutputSim <: MosaikAPIv3.Simulator
    models::Dict{Symbol,NamedTuple{(:type, :count),Tuple{DataType,Int}}} = Dict(:Output_Julia => (type = Nothing, count = 0))
    entities::Dict{String,String} = Dict()
    mosaik_remote::MosaikAPIv3.MosaikRemote
    sim_id::String
end

function MosaikAPIv3.init(
    ::Type{OutputSim},
    mosaik_remote::MosaikAPIv3.MosaikRemote,
    sim_id::String,
    time_resolution::Float64,
)
    @info "I'm an output simulator $sim_id, running with time resolution $time_resolution."
    OutputSim(;
        mosaik_remote = mosaik_remote,
        sim_id = sim_id,
    )
end

MosaikAPIv3.meta(::OutputSim) = MosaikMeta(;
    type = MosaikAPIv3.time_based,
    models = Dict(
        :Output_Julia => Model(;
            params = [],
            attrs = [:value]
        ),
    ),
    extra_methods = [get_dict],
)

function get_dict(sim::OutputSim)
    sim.entities
end

function MosaikAPIv3.create!(sim::OutputSim, num::Int64, model::String)
    model_type, count = sim.models[Symbol(model)]
    new_entities = MosaikAPIv3.CreateResult[]
    for i ∈ (count+1):(count+num)
        eid = "$model-$i"
        sim.entities[eid] = eid
        push!(new_entities, CreateResult(eid = eid, type = model))
    end
    sim.models[Symbol(model)] = (type = model_type, count = (count + num))
    new_entities
end

function MosaikAPIv3.setup_done!(::OutputSim)
    @info "Setup done!"
end

function MosaikAPIv3.step!(sim::OutputSim, time, inputs, max_advance)
    time + 1
end

function MosaikAPIv3.finalize!(::OutputSim)
    @info "Finalizing OutputSim"
end

MosaikAPIv3.run_server(OutputSim)