using MosaikAPIv3
using Test
using Logging

mutable struct SetEventSim <: MosaikAPIv3.Simulator
    models::Dict{Symbol,NamedTuple{(:type, :count),Tuple{DataType,Int}}}
    entities::Dict{String,String}
    event_happened::Bool
    mosaik_remote::MosaikAPIv3.MosaikRemote
    sim_id::String
    function SetEventSim()
        new(Dict(), Dict(), false)
    end
end

function MosaikAPIv3.init(
    ::Type{SetEventSim},
    mosaik_remote::MosaikAPIv3.MosaikRemote,
    sim_id::String,
    time_resolution::Float64,
)
    my_sim = SetEventSim()
    my_sim.mosaik_remote = mosaik_remote
    my_sim.models[Symbol("AsyncRequestsTest")] = (type = SetEventSim, count = 0)
    my_sim.sim_id = sim_id
    my_sim
end

MosaikAPIv3.meta(::SetEventSim) = MosaikMeta(;
    type = MosaikAPIv3.time_based,
    models = Dict(
        :AsyncRequestsTest => Model(;
            public = true,
            params = [], 
            attrs = [:value]),
    ),
)

function MosaikAPIv3.create!(my_sim::SetEventSim, num::Int64, model::String)
    model_type, count = my_sim.models[Symbol(model)]
    new_entities = MosaikAPIv3.CreateResult[]
    for i ∈ (count+1):(count+num)
        eid = "$model-$i"
        my_sim.entities[eid] = eid
        push!(new_entities, CreateResult(eid = eid, type = model))
    end
    my_sim.models[Symbol(model)] = (type = model_type, count = (count + num))
    new_entities
end

function MosaikAPIv3.step!(my_sim::SetEventSim, time, inputs, max_advance)
    time + 1
end

function MosaikAPIv3.finalize!(::SetEventSim)
    @info "Finalizing async requests"
end

MosaikAPIv3.run_server(SetEventSim)